--[[
	Plugin: mwse_PoisonCrafting.esp
--]]

local this = {
	config = {}
}

-- Config

function this.loadConfig()
	local cfg = json.loadfile("mwse_PoisonCrafting")

	if not cfg then
		-- defaults
		cfg = {
			msgPoisonApplied = true,
			msgReadyPoison = true,
			poisonHotkey = 18,
			useBaseStats = true,
			useBonusProgress = true,
			useLabels = true,
			usePoisonRecovery = true,
			version = 1,
		}
	end

	this.config = cfg
end


function this.saveConfig()
	this.loadLabels() -- ensures labels are reloaded after changes
	json.savefile("mwse_PoisonCrafting", this.config, {indent=true})
end


-- Interface

function this.prepareHUD()
	local scale = mge.getUIScale()
	local width = mge.getScreenWidth()
	local height = mge.getScreenHeight()

	mge.loadHUD{
		hud = "g7a_PoisonHUD",
		texture = "MFR/hud/PoisonDrop.dds",
	}
	mge.scaleHUD{
		hud = "g7a_PoisonHUD",
		x = scale,
		y = scale,
	}
	mge.positionHUD{
		hud = "g7a_PoisonHUD",
		x = scale * 110,
		y = height - scale * 64,
	}
end


-- Labels

do -- applyLabel
	local assets = {icon=".tga", model=".nif"}
	local qualities = {"exclusive", "quality", "fresh", "standard", "cheap", "bargain"}

	function this.applyLabel(potion)
		for asset, suffix in pairs(assets) do
			local current = potion[asset]:lower()
			for _, quality in pairs(qualities) do
				if current:find(quality) then
					local effect = potion.effects[1].id
					potion[asset] = "MFR\\msc\\" .. quality .. "_" .. effect .. suffix
					break
				end
			end
		end
	end
end


do -- clearLabel
	local assets = {
		icon = {
			bargain = "m\\tx_potion_bargain_01.dds",
			cheap = "m\\tx_potion_cheap_01.dds",
			exclusive = "m\\tx_potion_exclusive_01.dds",
			fresh = "m\\tx_potion_fresh_01.dds",
			quality = "m\\tx_potion_quality_01.dds",
			standard = "m\\tx_potion_standard_01.dds",
		},
		model = {
			bargain = "m\\misc_potion_bargain_01.nif",
			cheap = "m\\misc_potion_cheap_01.nif",
			exclusive = "m\\misc_potion_exclusive_01.nif",
			fresh = "m\\misc_potion_fresh_01.nif",
			quality = "m\\misc_potion_quality_01.nif",
			standard = "m\\misc_potion_standard_01.nif",
		},
	}
	function this.clearLabel(potion)
		for asset, qualities in pairs(assets) do
			local current = potion[asset]:lower()
			for quality, filename in pairs(qualities) do
				if current:find(quality) then
					potion[asset] = filename
					break
				end
			end
		end
	end
end


function this.loadLabels()
	local useLabels = this.config.useLabels

	for potion in tes3.iterateObjects(tes3.objectType.alchemy) do
		local isLabelled = (
			potion.model:lower():find("^MFR\\msc\\")
			and potion.icon:lower():find("^MFR\\msc\\")
		)
		if not useLabels and isLabelled then
			this.clearLabel(potion)
		elseif useLabels and not isLabelled then
			this.applyLabel(potion)
		end
	end
end


-- Compatibility

function this.confirmMCP()
	-- Confirm that some required MCP features are enabled.

	local labelFix = tes3.hasCodePatchFeature(145)
	local usageFix = tes3.hasCodePatchFeature(166)
	if (labelFix and usageFix) then
		return
	end

	local function warning()
		tes3.messageBox{
			message = (
				"MWSE_PoisonCrafting.esp\n\n"
				.."The required MCP features for this mod are not enabled!\n\n"
				.."Please enable the following:\n\n"
				.."Game mechanics -> Quality-based potion icons/models\n\n"
				.."Mod specific -> Scriptable potion use"
			),
			buttons = {"Ok"},
		}
	end

	event.register("menuEnter", warning, {doOnce = true})
end


function this.equip(args)
	-- compatibility for controlled consumption
	-- prevents enemy poisons causing cooldowns
	local cc = include("nc.consume.interop")

	if not cc then
		-- controlled consumption not installed
		this.equip = mwscript.equip
	else
		-- bypass controlled consumption checks
		function this.equip(args)
			local bool = cc.skipNextConsumptionCheck
			cc.skipNextConsumptionCheck = true
			mwscript.equip(args)
			cc.skipNextConsumptionCheck = bool
		end
		print("[g7a] Controlled Consumption Detected")
	end

	this.equip(args)
end


-- Menus

local parent = nil

do -- genericMenu
	local function resolve(x)
		return type(x) == "function" and x() or x
	end

	function this.genericMenu(t)

		function t.callback(e)
			local action = (
				t.actions[e.button+1]
				or this.configMain
			)
			action()
		end

		function t.menu()
			parent = t.menu
			tes3.messageBox{
				message = resolve(t.message),
				buttons = resolve(t.buttons),
				callback = t.callback,
			}
		end

		return t.menu
	end
end

do -- booleanMenu
	local bool = {[0]=false, true}

	local buttons = {
		[false] = {" ()", "", ""},
		[true]  = {"", " ()", ""},
	}

	function this.booleanMenu(t)

		function t.callback(e)
			if e.button == 2 then
				parent()
			else
				this.config[t.setting] = bool[e.button]
				t.menu()
			end
		end

		function t.menu()
			tes3.messageBox{
				message = t.message,
				buttons = buttons[this.config[t.setting]],
				callback = t.callback,
			}
		end

		return t.menu
	end
end

do -- hotkey info
	local buttons = {
		[-1]  = {"Alt", "Shift", "Ctrl", ""},
		[18]  = {"Alt ()", "Shift", "Control", ""},
		[160] = {"Alt", "Shift ()", "Control", ""},
		[162] = {"Alt", "Shift", "Control ()", ""},
	}
	function this.getHotkeyButtons()
		local b = buttons[this.config.poisonHotkey]
		return b or buttons[-1]
	end

	this.getHotkeyActions = {}
	for i, v in pairs{18, 160, 162} do
		this.getHotkeyActions[i] = (
			function()
				this.config.poisonHotkey = v
				this.configHotkey()
			end
		)
	end
end


this.configLabels = this.booleanMenu{
	message = (
		"            .\n\n"
		.."      ,     ."
	),
	setting = "useLabels",
}


this.configRecovery = this.booleanMenu{
	message = "      .",
	setting = "usePoisonRecovery",
}


this.configBaseStats = this.booleanMenu{
	message = (
		"      ."
		.."    ,       ."
	),
	setting = "useBaseStats",
}


this.configBonusProgress = this.booleanMenu{
	message = (
		"       ,   "
		.."     .    ,  ,"
		.."       10 ."
	),
	setting = "useBonusProgress",
}


this.configReadyPoison = this.booleanMenu{
	message = "        .",
	setting = "msgReadyPoison",
}


this.configPoisonApplied = this.booleanMenu{
	message = "         .",
	setting = "msgPoisonApplied",
}


this.configMessageBoxes = this.genericMenu{
	message = (
		"       "
		.."    .  ,   ."
	),
	buttons = {
		" ",
		" ",
		""
	},
	actions = {
		this.configReadyPoison,
		this.configPoisonApplied,
	},
}


this.configHotkey = this.genericMenu{
	message = "       .         .",
	buttons = this.getHotkeyButtons,
	actions = this.getHotkeyActions,
}


this.configMain = this.genericMenu{
	message = "   ?",
	buttons = {
		"   ",
		" ",
		" ",
		"  ",
		"  ",
		" ",
		"",
	},
	actions = {
		this.configHotkey,
		this.configRecovery,
		this.configLabels,
		this.configBaseStats,
		this.configBonusProgress,
		this.configMessageBoxes,
		this.saveConfig,
	},
}


-- Events

function this.onLoaded(e)
	-- ensure data table exists
	local data = tes3.getPlayerRef().data
	data.g7a = data.g7a or {}

	-- also do projectile table
	data.g7a.projectiles = data.g7a.projectiles or {}

	-- create a public shortcut
	this.data = data.g7a
end


function this.register()
	event.register("loaded", this.onLoaded)
end


return this
